@@ -38,36 +38,38 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/constitute.h"
-#include "magick/delegate.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/geometry.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/layer.h"
-#include "magick/list.h"
-#include "magick/log.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/opencl.h"
-#include "magick/option.h"
-#include "magick/pixel-accessor.h"
-#include "magick/profile.h"
-#include "magick/property.h"
-#include "magick/quantum-private.h"
-#include "magick/resource_.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/module.h"
-#include "magick/transform.h"
-#include "magick/utility.h"
-#include "magick/xml-tree.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/constitute.h"
+#include "MagickCore/delegate.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/geometry.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/layer.h"
+#include "MagickCore/list.h"
+#include "MagickCore/log.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/opencl.h"
+#include "MagickCore/option.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/profile.h"
+#include "MagickCore/property.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/resource_.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/string-private.h"
+#include "MagickCore/module.h"
+#include "MagickCore/transform.h"
+#include "MagickCore/utility.h"
+#include "MagickCore/xml-tree.h"
+#include "MagickCore/xml-tree-private.h"
 #if defined(MAGICKCORE_RAW_R_DELEGATE)
 #include <libraw.h>
 #endif
@@ -103,46 +105,50 @@
 #if defined(MAGICKCORE_WINDOWS_SUPPORT) && defined(MAGICKCORE_OPENCL_SUPPORT)
 static void InitializeDcrawOpenCL(ExceptionInfo *exception)
 {
-  MagickCLEnv
-    clEnv;
+  MagickBooleanType
+    opencl_disabled;
+
+  MagickCLDevice
+    *devices;
+
+  size_t
+    length;
+
+  ssize_t
+    i;
 
   (void) SetEnvironmentVariable("DCR_CL_PLATFORM",NULL);
   (void) SetEnvironmentVariable("DCR_CL_DEVICE",NULL);
   (void) SetEnvironmentVariable("DCR_CL_DISABLED",NULL);
-  clEnv=GetDefaultOpenCLEnv();
-  if (InitOpenCLEnv(clEnv,exception) != MagickFalse)
+  if (GetOpenCLEnabled() == MagickFalse)
     {
-      char
-        *name;
-
-      MagickBooleanType
-        opencl_disabled;
-
-      GetMagickOpenCLEnvParam(clEnv,MAGICK_OPENCL_ENV_PARAM_OPENCL_DISABLED,
-        sizeof(MagickBooleanType),&opencl_disabled,exception);
-      if (opencl_disabled != MagickFalse)
-        {
-          (void)SetEnvironmentVariable("DCR_CL_DISABLED","1");
-          return;
-        }
-      GetMagickOpenCLEnvParam(clEnv,MAGICK_OPENCL_ENV_PARAM_PLATFORM_VENDOR,
-        sizeof(char *),&name,exception);
-      if (name != (char *) NULL)
-      {
-        (void) SetEnvironmentVariable("DCR_CL_PLATFORM",name);
-        name=RelinquishMagickMemory(name);
-      }
-      GetMagickOpenCLEnvParam(clEnv,MAGICK_OPENCL_ENV_PARAM_DEVICE_NAME,
-        sizeof(char *),&name,exception);
-      if (name != (char *) NULL)
-      {
-        (void) SetEnvironmentVariable("DCR_CL_DEVICE",name);
-        name=RelinquishMagickMemory(name);
-      }
+      (void) SetEnvironmentVariable("DCR_CL_DISABLED","1");
+      return;
     }
+  devices=GetOpenCLDevices(&length,exception);
+  if (devices == (MagickCLDevice *) NULL)
+    return;
+  for (i=0; i < (ssize_t) length; i++)
+  {
+    const char
+      *name;
+
+    MagickCLDevice
+      device;
+
+    device=devices[i];
+    if (GetOpenCLDeviceEnabled(device) == MagickFalse)
+      continue;
+    name=GetOpenCLDeviceVendorName(device);
+    if (name != (const char *) NULL)
+      (void) SetEnvironmentVariable("DCR_CL_PLATFORM",name);
+    name=GetOpenCLDeviceName(device);
+    if (name != (const char *) NULL)
+      (void) SetEnvironmentVariable("DCR_CL_DEVICE",name);
+    return;
+  }
 }
 #else
-#if !defined(MAGICKCORE_RAW_R_DELEGATE)
 static void InitializeDcrawOpenCL(ExceptionInfo *magick_unused(exception))
 {
   magick_unreferenced(exception);
@@ -151,68 +157,187 @@ static void InitializeDcrawOpenCL(ExceptionInfo *magick_unused(exception))
 #endif
 }
 #endif
-#endif
 
 #if defined(MAGICKCORE_RAW_R_DELEGATE)
-static void SetDNGProperties(Image *image,const libraw_data_t *raw_info)
+static void SetDNGProperties(Image *image,const libraw_data_t *raw_info,
+  ExceptionInfo *exception)
 {
   char
-    property[MagickPathExtent],
     timestamp[MagickPathExtent];
 
-  (void) SetImageProperty(image,"dng:make",raw_info->idata.make);
-  (void) SetImageProperty(image,"dng:camera.model.name",raw_info->idata.model);
+  (void) SetImageProperty(image,"dng:make",raw_info->idata.make,exception);
+  (void) SetImageProperty(image,"dng:camera.model.name",raw_info->idata.model,
+    exception);
   (void) FormatMagickTime(raw_info->other.timestamp,MagickPathExtent,timestamp);
-  (void) SetImageProperty(image,"dng:create.date",timestamp);
-  (void) SetImageProperty(image,"dng:f.number",property);
-  (void) FormatLocaleString(property,MagickPathExtent,"%0.1f",
+  (void) SetImageProperty(image,"dng:create.date",timestamp,exception);
+  (void) FormatImageProperty(image,"dng:iso.setting","%0.1f",
     raw_info->other.iso_speed);
 #if LIBRAW_COMPILE_CHECK_VERSION_NOTLESS(0,18)
-  (void) SetImageProperty(image,"dng:software",raw_info->idata.software);
+  (void) SetImageProperty(image,"dng:software",raw_info->idata.software,
+    exception);
   if (*raw_info->shootinginfo.BodySerial != '\0')
     (void) SetImageProperty(image,"dng:serial.number",
-      raw_info->shootinginfo.BodySerial);
-  (void) FormatLocaleString(property,MagickPathExtent,"1/%0.1f",
-    PerceptibleReciprocal(raw_info->other.shutter));
-  (void) SetImageProperty(image,"dng:exposure.time",property);
-  (void) FormatLocaleString(property,MagickPathExtent,"%0.1f",
+      raw_info->shootinginfo.BodySerial,exception);
+  (void) FormatImageProperty(image,"dng:exposure.time","1/%0.1f",
+    1.0/raw_info->other.shutter);
+  (void) FormatImageProperty(image,"dng:f.number","%0.1f",
     raw_info->other.aperture);
-  (void) SetImageProperty(image,"dng:iso.setting",property);
-  (void) FormatLocaleString(property,MagickPathExtent,"%0.1f",
+  (void) FormatImageProperty(image,"dng:max.aperture.value","%0.1f",
     raw_info->lens.EXIF_MaxAp);
-  (void) SetImageProperty(image,"dng:max.aperture.value",property);
-  (void) FormatLocaleString(property,MagickPathExtent,"%0.1f",
+  (void) FormatImageProperty(image,"dng:ocal.length","%0.1f",
     raw_info->other.focal_len);
-  (void) SetImageProperty(image,"dng:focal.length",property);
-  (void) FormatLocaleString(property,MagickPathExtent,"%f %f %f %f",
+  (void) FormatImageProperty(image,"dng:wb.rb.levels","%f %f %f %f",
     raw_info->color.cam_mul[0],raw_info->color.cam_mul[2],
     raw_info->color.cam_mul[1],raw_info->color.cam_mul[3]);
-  (void) SetImageProperty(image,"dng:wb.rb.levels",property);
-  (void) FormatLocaleString(property,MagickPathExtent,
-    "%0.1f-%0.1fmm f/%0.1f-%0.1f",raw_info->lens.makernotes.MinFocal,
-    raw_info->lens.makernotes.MaxFocal,raw_info->lens.makernotes.MaxAp4MinFocal,
+  (void) SetImageProperty(image,"dng:lens.type",
+    raw_info->lens.makernotes.LensFeatures_suf,exception);
+  (void) FormatImageProperty(image,"dng:lens","%0.1f-%0.1fmm f/%0.1f-%0.1f",
+    raw_info->lens.makernotes.MinFocal,raw_info->lens.makernotes.MaxFocal,
+    raw_info->lens.makernotes.MaxAp4MinFocal,
     raw_info->lens.makernotes.MaxAp4MaxFocal);
-  (void) SetImageProperty(image,"dng:lens",property);
-  (void) FormatLocaleString(property,MagickPathExtent,"%0.2f",
+  (void) FormatImageProperty(image,"dng:lens.f.stops","%0.2f",
     raw_info->lens.makernotes.LensFStops);
-  (void) SetImageProperty(image,"dng:lens.f.stops",property);
-  (void) FormatLocaleString(property,MagickPathExtent,"%0.1f mm",
+  (void) FormatImageProperty(image,"dng:min.focal.length","%0.1f mm",
     raw_info->lens.makernotes.MinFocal);
-  (void) SetImageProperty(image,"dng:min.focal.length",property);
-  (void) FormatLocaleString(property,MagickPathExtent,"%0.1f mm",
+  (void) FormatImageProperty(image,"dng:max.focal.length","%0.1f mm",
     raw_info->lens.makernotes.MaxFocal);
-  (void) SetImageProperty(image,"dng:max.focal.length",property);
-  (void) SetImageProperty(image,"dng:max.aperture.at.min.focal",property);
-  (void) FormatLocaleString(property,MagickPathExtent,"%0.1f",
+  (void) FormatImageProperty(image,"dng:max.aperture.at.min.focal","%0.1f",
+    raw_info->lens.makernotes.MaxAp4MinFocal);
+  (void) FormatImageProperty(image,"dng:max.aperture.at.max.focal","%0.1f",
     raw_info->lens.makernotes.MaxAp4MaxFocal);
-  (void) SetImageProperty(image,"dng:max.aperture.at.max.focal",property);
-  (void) FormatLocaleString(property,MagickPathExtent,"%d mm",
+  (void) FormatImageProperty(image,"dng:focal.length.in.35mm.format","%d mm",
     raw_info->lens.FocalLengthIn35mmFormat);
-  (void) SetImageProperty(image,"dng:focal.length.in.35mm.format",property);
 #endif
 }
 #endif
 
+static Image *InvokeDNGDelegate(const ImageInfo *image_info,Image *image,
+  ExceptionInfo *exception)
+{
+  ExceptionInfo
+    *sans_exception;
+
+  ImageInfo
+    *read_info;
+
+  /*
+    Convert DNG to PPM with delegate.
+  */
+  (void) DestroyImageList(image);
+  InitializeDcrawOpenCL(exception);
+  image=AcquireImage(image_info,exception);
+  read_info=CloneImageInfo(image_info);
+  SetImageInfoBlob(read_info,(void *) NULL,0);
+  (void) InvokeDelegate(read_info,image,"dng:decode",(char *) NULL,exception);
+  image=DestroyImage(image);
+  (void) FormatLocaleString(read_info->filename,MagickPathExtent,"%s.png",
+    read_info->unique);
+  sans_exception=AcquireExceptionInfo();
+  image=ReadImage(read_info,sans_exception);
+  sans_exception=DestroyExceptionInfo(sans_exception);
+  if (image == (Image *) NULL)
+    {
+      (void) FormatLocaleString(read_info->filename,MagickPathExtent,"%s.ppm",
+        read_info->unique);
+      image=ReadImage(read_info,exception);
+    }
+  (void) RelinquishUniqueFileResource(read_info->filename);
+  if (image != (Image *) NULL)
+    {
+      char
+        filename[MagickPathExtent],
+        *xml;
+
+      ExceptionInfo
+        *sans;
+
+      (void) CopyMagickString(image->magick,read_info->magick,
+        MagickPathExtent);
+      (void) FormatLocaleString(filename,MagickPathExtent,"%s.ufraw",
+        read_info->unique);
+      sans=AcquireExceptionInfo();
+      xml=FileToString(filename,MagickPathExtent,sans);
+      (void) RelinquishUniqueFileResource(filename);
+      if (xml != (char *) NULL)
+        {
+          XMLTreeInfo
+            *ufraw;
+
+          /*
+            Inject.
+          */
+          ufraw=NewXMLTree(xml,sans);
+          if (ufraw != (XMLTreeInfo *) NULL)
+            {
+              char
+                *content,
+                property[MagickPathExtent];
+
+              const char
+                *tag;
+
+              XMLTreeInfo
+                *next;
+
+              if (image->properties == (void *) NULL)
+                image->properties=NewSplayTree(CompareSplayTreeString,
+                  RelinquishMagickMemory,RelinquishMagickMemory);
+              next=GetXMLTreeChild(ufraw,(const char *) NULL);
+              while (next != (XMLTreeInfo *) NULL)
+              {
+                tag=GetXMLTreeTag(next);
+                if (tag == (char *) NULL)
+                  tag="unknown";
+                (void) FormatLocaleString(property,MagickPathExtent,"dng:%s",
+                  tag);
+                content=ConstantString(GetXMLTreeContent(next));
+                StripString(content);
+                if ((LocaleCompare(tag,"log") != 0) &&
+                    (LocaleCompare(tag,"InputFilename") != 0) &&
+                    (LocaleCompare(tag,"OutputFilename") != 0) &&
+                    (LocaleCompare(tag,"OutputType") != 0) &&
+                    (strlen(content) != 0))
+                  (void) AddValueToSplayTree((SplayTreeInfo *)
+                    image->properties,ConstantString(property),content);
+                next=GetXMLTreeSibling(next);
+              }
+              ufraw=DestroyXMLTree(ufraw);
+            }
+          xml=DestroyString(xml);
+        }
+      sans=DestroyExceptionInfo(sans);
+    }
+  read_info=DestroyImageInfo(read_info);
+  return(image);
+}
+
+#if defined(MAGICKCORE_RAW_R_DELEGATE)
+static void SetLibRawParams(const ImageInfo *image_info,Image *image,
+  libraw_data_t *raw_info)
+{
+  const char
+    *option;
+
+  raw_info->params.output_bps=16;
+  option=GetImageOption(image_info,"dng:use_camera_wb");
+  if (option != (const char *) NULL)
+    raw_info->params.use_camera_wb=IsStringTrue(option);
+  option=GetImageOption(image_info,"dng:use_auto_wb");
+  if (option != (const char *) NULL)
+    raw_info->params.use_auto_wb=IsStringTrue(option);
+  option=GetImageOption(image_info,"dng:no_auto_bright");
+  if (option != (const char *) NULL)
+    raw_info->params.no_auto_bright=IsStringTrue(option);
+  option=GetImageOption(image_info,"dng:output_color");
+  if (option != (const char *) NULL)
+    {
+      raw_info->params.output_color=StringToInteger(option);
+      if (raw_info->params.output_color == 5)
+        image->colorspace=XYZColorspace;
+    }
+}
+#endif
+
 static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   Image
@@ -231,7 +356,7 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -239,6 +364,8 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
       return((Image *) NULL);
     }
   (void) CloseBlob(image);
+  if (LocaleCompare(image_info->magick,"DCRAW") == 0)
+    return(InvokeDNGDelegate(image_info,image,exception));
 #if defined(MAGICKCORE_RAW_R_DELEGATE)
   {
     int
@@ -268,8 +395,6 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
         libraw_close(raw_info);
         return(DestroyImageList(image));
       }
-    raw_info->params.use_camera_wb=IsStringTrue(GetImageOption(image_info,
-      "dng:use-camera-wb"));
 #if defined(MAGICKCORE_WINDOWS_SUPPORT) && defined(_MSC_VER) && (_MSC_VER > 1310)
     {
       wchar_t
@@ -300,7 +425,7 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
         libraw_close(raw_info);
         return(image);
       }
-    status=SetImageExtent(image,image->columns,image->rows);
+    status=SetImageExtent(image,image->columns,image->rows,exception);
     if (status == MagickFalse)
       {
         libraw_close(raw_info);
@@ -314,7 +439,7 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
         libraw_close(raw_info);
         return(DestroyImageList(image));
       }
-    raw_info->params.output_bps=16;
+    SetLibRawParams(image_info,image,raw_info);
     errcode=libraw_dcraw_process(raw_info);
     if (errcode != LIBRAW_SUCCESS)
       {
@@ -324,7 +449,7 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
         return(DestroyImageList(image));
       }
     raw_image=libraw_dcraw_make_mem_image(raw_info,&errcode);
-    if ((errcode != LIBRAW_SUCCESS) || 
+    if ((errcode != LIBRAW_SUCCESS) ||
         (raw_image == (libraw_processed_image_t *) NULL) ||
         (raw_image->type != LIBRAW_IMAGE_BITMAP) || (raw_image->bits != 16) ||
         (raw_image->colors < 3) || (raw_image->colors > 4))
@@ -339,11 +464,7 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->columns=raw_image->width;
     image->rows=raw_image->height;
     image->depth=raw_image->bits;
-    image->page.width=raw_info->sizes.width;
-    image->page.height=raw_info->sizes.height;
-    image->page.x=raw_info->sizes.left_margin;
-    image->page.y=raw_info->sizes.top_margin;
-    status=SetImageExtent(image,image->columns,image->rows);
+    status=SetImageExtent(image,image->columns,image->rows,exception);
     if (status == MagickFalse)
       {
         libraw_dcraw_clear_mem(raw_image);
@@ -353,23 +474,23 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
     p=(unsigned short *) raw_image->data;
     for (y=0; y < (ssize_t) image->rows; y++)
     {
-      register PixelPacket
+      register Quantum
         *q;
 
       register ssize_t
         x;
 
       q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-      if (q == (PixelPacket *) NULL)
+      if (q == (Quantum *) NULL)
         break;
       for (x=0; x < (ssize_t) image->columns; x++)
       {
-        SetPixelRed(q,ScaleShortToQuantum(*p++));
-        SetPixelGreen(q,ScaleShortToQuantum(*p++));
-        SetPixelBlue(q,ScaleShortToQuantum(*p++));
+        SetPixelRed(image,ScaleShortToQuantum(*p++),q);
+        SetPixelGreen(image,ScaleShortToQuantum(*p++),q);
+        SetPixelBlue(image,ScaleShortToQuantum(*p++),q);
         if (raw_image->colors > 3)
-          SetPixelAlpha(q,ScaleShortToQuantum(*p++));
-        q++;
+          SetPixelAlpha(image,ScaleShortToQuantum(*p++),q);
+        q+=GetPixelChannels(image);
       }
       if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
@@ -391,7 +512,7 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
           raw_info->color.profile_length);
         if (profile != (StringInfo *) NULL)
           {
-            SetImageProfile(image,"ICC",profile);
+            SetImageProfile(image,"ICC",profile,exception);
             profile=DestroyStringInfo(profile);
           }
       }
@@ -402,114 +523,17 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
           raw_info->idata.xmplen);
         if (profile != (StringInfo *) NULL)
           {
-            SetImageProfile(image,"XMP",profile);
+            SetImageProfile(image,"XMP",profile,exception);
             profile=DestroyStringInfo(profile);
           }
       }
 #endif
-    SetDNGProperties(image,raw_info);
+    SetDNGProperties(image,raw_info,exception);
     libraw_close(raw_info);
     return(image);
   }
 #else
-  {
-    ExceptionInfo
-      *sans_exception;
-
-    ImageInfo
-      *read_info;
-
-    /*
-      Convert DNG to PPM with delegate.
-    */
-    (void) DestroyImageList(image);
-    InitializeDcrawOpenCL(exception);
-    image=AcquireImage(image_info);
-    read_info=CloneImageInfo(image_info);
-    SetImageInfoBlob(read_info,(void *) NULL,0);
-    (void) InvokeDelegate(read_info,image,"dng:decode",(char *) NULL,exception);
-    image=DestroyImage(image);
-    (void) FormatLocaleString(read_info->filename,MaxTextExtent,"%s.png",
-      read_info->unique);
-    sans_exception=AcquireExceptionInfo();
-    image=ReadImage(read_info,sans_exception);
-    sans_exception=DestroyExceptionInfo(sans_exception);
-    if (image == (Image *) NULL)
-      {
-        (void) FormatLocaleString(read_info->filename,MaxTextExtent,"%s.ppm",
-          read_info->unique);
-        image=ReadImage(read_info,exception);
-      }
-    (void) RelinquishUniqueFileResource(read_info->filename);
-    if (image != (Image *) NULL)
-      {
-        char
-          filename[MaxTextExtent],
-          *xml;
-
-        ExceptionInfo
-          *sans;
-
-        (void) CopyMagickString(image->magick,read_info->magick,MaxTextExtent);
-        (void) FormatLocaleString(filename,MaxTextExtent,"%s.ufraw",
-          read_info->unique);
-        sans=AcquireExceptionInfo();
-        xml=FileToString(filename,MaxTextExtent,sans);
-        (void) RelinquishUniqueFileResource(filename);
-        if (xml != (char *) NULL)
-          {
-            XMLTreeInfo
-              *ufraw;
-
-            /*
-              Inject.
-            */
-            ufraw=NewXMLTree(xml,sans);
-            if (ufraw != (XMLTreeInfo *) NULL)
-              {
-                char
-                  *content,
-                  property[MaxTextExtent];
-
-                const char
-                  *tag;
-
-                XMLTreeInfo
-                  *next;
-
-                if (image->properties == (void *) NULL)
-                  ((Image *) image)->properties=NewSplayTree(
-                    CompareSplayTreeString,RelinquishMagickMemory,
-                    RelinquishMagickMemory);
-                next=GetXMLTreeChild(ufraw,(const char *) NULL);
-                while (next != (XMLTreeInfo *) NULL)
-                {
-                  tag=GetXMLTreeTag(next);
-                  if (tag == (char *) NULL)
-                    tag="unknown";
-                  (void) FormatLocaleString(property,MaxTextExtent,"dng:%s",
-                    tag);
-                  content=ConstantString(GetXMLTreeContent(next));
-                  StripString(content);
-                  if ((LocaleCompare(tag,"log") != 0) &&
-                      (LocaleCompare(tag,"InputFilename") != 0) &&
-                      (LocaleCompare(tag,"OutputFilename") != 0) &&
-                      (LocaleCompare(tag,"OutputType") != 0) &&
-                      (strlen(content) != 0))
-                    (void) AddValueToSplayTree((SplayTreeInfo *)
-                      ((Image *) image)->properties,ConstantString(property),
-                      content);
-                  next=GetXMLTreeSibling(next);
-                }
-                ufraw=DestroyXMLTree(ufraw);
-              }
-            xml=DestroyString(xml);
-          }
-        sans=DestroyExceptionInfo(sans);
-      }
-    read_info=DestroyImageInfo(read_info);
-    return(image);
-  }
+  return(InvokeDNGDelegate(image_info,image,exception));
 #endif
 }
 
@@ -541,220 +565,157 @@ ModuleExport size_t RegisterDNGImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("3FR");
+  entry=AcquireMagickInfo("DNG","3FR","Hasselblad CFV/H3D39II");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Hasselblad CFV/H3D39II");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("ARW");
+  entry=AcquireMagickInfo("DNG","ARW","Sony Alpha Raw Image Format");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Sony Alpha Raw Image Format");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("DNG");
+  entry=AcquireMagickInfo("DNG","DNG","Digital Negative");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Digital Negative");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("CR2");
+  entry=AcquireMagickInfo("DNG","CR2","Canon Digital Camera Raw Image Format");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Canon Digital Camera Raw Image Format");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("CR3");
+  entry=AcquireMagickInfo("DNG","CR3","Canon Digital Camera Raw Image Format");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Canon Digital Camera Raw Image Format");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("CRW");
+  entry=AcquireMagickInfo("DNG","CRW","Canon Digital Camera Raw Image Format");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Canon Digital Camera Raw Image Format");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("DCR");
+  entry=AcquireMagickInfo("DNG","DCR","Kodak Digital Camera Raw Image File");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Kodak Digital Camera Raw Image File");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("ERF");
+  entry=AcquireMagickInfo("DNG","DCRAW","Raw Photo Decoder (dcraw)");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Epson Raw Format");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("IIQ");
+  entry=AcquireMagickInfo("DNG","ERF","Epson RAW Format");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Phase One Raw Image Format");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("KDC");
+  entry=AcquireMagickInfo("DNG","IIQ","Phase One Raw Image Format");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Kodak Digital Camera Raw Image Format");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("K25");
+  entry=AcquireMagickInfo("DNG","KDC","Kodak Digital Camera Raw Image Format");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Kodak Digital Camera Raw Image Format");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("MEF");
+  entry=AcquireMagickInfo("DNG","K25","Kodak Digital Camera Raw Image Format");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Mamiya Raw Image File");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("MRW");
+  entry=AcquireMagickInfo("DNG","MEF","Mamiya Raw Image File");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Sony (Minolta) Raw Image File");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("NEF");
+  entry=AcquireMagickInfo("DNG","MRW","Sony (Minolta) Raw Image File");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Nikon Digital SLR Camera Raw Image File");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("NRW");
+  entry=AcquireMagickInfo("DNG","NEF",
+    "Nikon Digital SLR Camera Raw Image File");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Nikon Digital SLR Camera Raw Image File");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("ORF");
+  entry=AcquireMagickInfo("DNG","NRW",
+    "Nikon Digital SLR Camera Raw Image File");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Olympus Digital Camera Raw Image File");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("PEF");
+  entry=AcquireMagickInfo("DNG","ORF","Olympus Digital Camera Raw Image File");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Pentax Electronic File");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("RAF");
+  entry=AcquireMagickInfo("DNG","PEF","Pentax Electronic File");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Fuji CCD-RAW Graphic File");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("RAW");
+  entry=AcquireMagickInfo("DNG","RAF","Fuji CCD-RAW Graphic File");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Raw");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("RMF");
+  entry=AcquireMagickInfo("DNG","RAW","Raw");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Raw Media Format");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("RW2");
+  entry=AcquireMagickInfo("DNG","RMF","Raw Media Format");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Panasonic Lumix Raw Image");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("SRF");
+  entry=AcquireMagickInfo("DNG","RW2","Panasonic Lumix Raw Image");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Sony Raw Format");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("SR2");
+  entry=AcquireMagickInfo("DNG","SRF","Sony Raw Format");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Sony Raw Format 2");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("X3F");
+  entry=AcquireMagickInfo("DNG","SR2","Sony Raw Format 2");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
+  entry->format_type=ExplicitFormatType;
+  (void) RegisterMagickInfo(entry);
+  entry=AcquireMagickInfo("DNG","X3F","Sigma Camera RAW Picture File");
+  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Sigma Camera RAW Picture File");
-  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -785,7 +746,6 @@ ModuleExport void UnregisterDNGImage(void)
   (void) UnregisterMagickInfo("SRF");
   (void) UnregisterMagickInfo("RW2");
   (void) UnregisterMagickInfo("RMF");
-  (void) UnregisterMagickInfo("RAW");
   (void) UnregisterMagickInfo("RAF");
   (void) UnregisterMagickInfo("PEF");
   (void) UnregisterMagickInfo("ORF");
